﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Portorechner
{
    public enum Produktartwerte { Brief, Paket};
    public enum Briefformat { C4, C5, C6 };
    public enum Paketformat { Standard, Sperrgut };


    public class  Postprodukt
    {
        
        protected Produktartwerte produktart;

        public Produktartwerte Produktart
        {
            get { return produktart; }
            set { produktart = value; }
        }
        protected double entfernungskm;

        public double Entfernungskm
        {
            get { return entfernungskm; }
            set { entfernungskm = value; }
        }
        protected double gewicht;

        public double Gewicht
        {
            get { return gewicht; }
            set { gewicht = value; }
        }

        public Postprodukt(Produktartwerte pProduktart, double pEntfernungskm, double pGewicht)
        {
            Produktart=pProduktart;
            Entfernungskm = pEntfernungskm;
            Gewicht = pGewicht;
        }
        public override string ToString()
        {
            return String.Format("Produktart: {0}, Porto: {1:n} €"
                        , produktart, (this.PortoBerechnen()/100));

        }
        public virtual double PortoBerechnen()
            // Anmerkung: Während in Java jede automatisch als 
            // "virtual" deklariert, muss dies in C# explizit 
            // erfolgen.
        {
            return 30 + entfernungskm * 0.05;
        }

    }

    public class Brief : Postprodukt
    {
        public Brief(double pEntfernungsKm, double pGewicht, Briefformat pFormat):base(Produktartwerte.Brief,pEntfernungsKm,pGewicht)
        {
            format = pFormat;
        }
        private Briefformat format;

        internal Briefformat Format
        {
            get { return format; }
            set { format = value; }
        }
        public override double PortoBerechnen()
        {
            double porto = base.PortoBerechnen();
            switch (format)
            {
                case Briefformat.C4: porto = porto * 1.4;
                    break;
                case Briefformat.C5: porto = porto * 1.2;
                    break;
                case Briefformat.C6: porto = porto * 1;
                    break;
            }
            if (gewicht > 100) porto = porto * 2;
            return porto;
        }
    }

    public class Paket : Postprodukt
    {
        private Paketformat format;

        public Paketformat Format
        {
            get { return format; }
            set { format = value; }
        }
        private double breite;

        public double Breite
        {
            get { return breite; }
            set { breite = value; }
        }
        private double laenge;

        public double Laenge
        {
            get { return laenge; }
            set { laenge = value; }
        }
        private double hoehe;

        public double Hoehe
        {
            get { return hoehe; }
            set { hoehe = value; }
        }
    
         public Paket(Paketformat pFormat, double pEntfernungsKm, double pGewicht, double pLaenge, double pBreite, double pHoehe):base(Produktartwerte.Paket,pEntfernungsKm,pGewicht)
        {
            format = pFormat;
            laenge = pLaenge;
            breite = pBreite;
            hoehe = pHoehe;
        }

         public double VolumenBerechnen()
         {
             return laenge*breite*hoehe;
         }

         public override double PortoBerechnen()
         {
             double porto=200;
             if (format == Paketformat.Sperrgut) porto = porto * 3;
             int n = (int)(Entfernungskm / 100);
             if (Entfernungskm > (n * 100)) { n = n + 1; }
             //
             int m = (int)(Gewicht / 1000);
             if (Gewicht > (m * 1000)) { m = m + 1; }
             porto = porto + n * 50 + m * 30 + 4 * 0.001 * VolumenBerechnen();
             return porto;
         }

    }
}
